// Package point imlements simple Point API wrapper for getting different info
package point

import (
	"bitbucket.org/skobkin/dumb-http-go"
	"encoding/json"
	"errors"
	"log"
	"net/http"
	"net/url"
	"strconv"
)

type PointClient struct {
	client simple_http.Client
	apiUrl string
	token  Token
}

// New creates new Point.im API client instance and initialize it with provided URL
func New(apiUrl string) PointClient {
	return PointClient{
		simple_http.Client{http.Client{}},
		apiUrl,
		Token{},
	}
}

// Login tries to log in to Point.im API and store authentication token on success. Token will be used automatically on next API calls.
func (c *PointClient) Login(login, password string) (Token, error) {
	var token Token

	data := url.Values{}
	data.Set("login", login)
	data.Set("password", password)

	body, reqErr := c.client.MakePostRequest(c.apiUrl+"login", data, nil)

	if reqErr != nil {
		return token, reqErr
	}

	jsonErr := json.Unmarshal(body, &token)

	if jsonErr != nil {
		log.Fatal(jsonErr)
		return token, errors.New("JSON deserialization error")
	}

	if token.Error != "" {
		return token, errors.New(token.Error)
	}

	c.token = token

	return token, nil
}

// GetRecentAllPostsPage gets most recent /all posts page
func (c *PointClient) GetRecentAllPostsPage() (Page, error) {
	var page Page

	if len(c.token.AuthToken) == 0 {
		log.Fatal("Can not get recent posts. Login first.")
		return page, errors.New("Login first")
	}

	headers := map[string]string{
		"Authorization": c.token.AuthToken,
	}

	body, reqErr := c.client.MakeGetRequest(c.apiUrl+"all", nil, &headers)

	if reqErr != nil {
		return page, errors.New("HTTP request error")
	}

	jsonErr := json.Unmarshal(body, &page)

	if jsonErr != nil {
		log.Fatal(jsonErr)
		return page, errors.New("JSON deserialization error")
	}

	// Move page.Error to response object
	if page.Error == "NotAuthorized" {
		log.Fatal("Please use correct login and password to fetch recent posts")
		return page, errors.New("Not authorized")
	}

	return page, nil
}

// GetNextAllPostsPageBeforeUid gets next /all page after provided post uid
func (c *PointClient) GetNextAllPostsPageBeforeUid(uid int) (Page, error) {
	var nextPage Page

	if len(c.token.AuthToken) == 0 {
		log.Fatal("Can not get recent posts. Login first.")
		return nextPage, errors.New("Login first")
	}

	data := url.Values{}
	data.Set("before", strconv.Itoa(uid))

	headers := map[string]string{
		"Authorization": c.token.AuthToken,
	}

	body, req_err := c.client.MakeGetRequest(c.apiUrl+"all", &data, &headers)

	if req_err != nil {
		return nextPage, errors.New("HTTP request error")
	}

	json_err := json.Unmarshal(body, &nextPage)

	if json_err != nil {
		log.Fatal(json_err)
		return nextPage, errors.New("JSON deserialization error")
	}

	// Move page.Error to response object
	if nextPage.Error == "NotAuthorized" {
		log.Fatal("Please use correct login and password to fetch recent posts")
		return nextPage, errors.New("Not authorized")
	}

	return nextPage, nil
}

// GetNextAllPostsPage gets next /all page after provided page
func (c *PointClient) GetNextAllPostsPage(page Page) (Page, error) {
	var nextPage Page

	if len(page.Posts) == 0 {
		return nextPage, errors.New("Page must have has_next=true and some posts")
	}

	if len(c.token.AuthToken) == 0 {
		log.Fatal("Can not get recent posts. Login first.")
		return nextPage, errors.New("Login first")
	}

	nextPage, err := c.GetNextAllPostsPageBeforeUid(page.Posts[len(page.Posts)-1].Uid)

	return nextPage, err
}
